from os import path, listdir
from pathlib import Path

from flask import Flask, render_template
from flask_frozen import Freezer


template_folder = path.abspath('./wiki')

app = Flask(__name__, template_folder=template_folder)
# app.config['FREEZER_BASE_URL'] = environ.get('CI_PAGES_URL')
app.config['FREEZER_DESTINATION'] = 'public'
app.config['FREEZER_RELATIVE_URLS'] = True
app.config['FREEZER_IGNORE_MIMETYPE_WARNINGS'] = True
freezer = Freezer(app)

@app.cli.command()
def freeze():
    freezer.freeze()

@app.cli.command()
def serve():
    freezer.run()

@freezer.register_generator
def pages():
    for filename in listdir(template_folder + "/pages/"):
        if filename.endswith('.html'):
            yield {'page': filename[:-5]}

@app.route('/')
def home():
    return render_template('pages/home.html',show_toc=False)


@app.route('/<page>/')
def pages(page):
    if page == 'team' or page == 'attributions' or page == "software" or page == "breseq" or page == "adcd":
        return render_template(str(Path('pages')) + '/' + page.lower() + '.html',show_toc=False)
    return render_template(str(Path('pages')) + '/' + page.lower() + '.html',show_toc=True)

@app.after_request
def add_security_headers(response):
    response.headers['Content-Security-Policy'] = (
        "default-src 'self'; "
        "frame-src 'self' https://www.youtube.com;"
    )
    return response

'''
@app.route('/breseq')
def breseq():
    return render_template('breseq', 'index.html')
'''

# Main Function, Runs at http://0.0.0.0:8080
if __name__ == "__main__":
    app.run(port=8080)