function translate_codon(codon) {
    let genetic_code = {
        'ATA':'Ile', 'ATC':'Ile', 'ATT':'Ile', 'ATG':'Met',
        'ACA':'Thr', 'ACC':'Thr', 'ACG':'Thr', 'ACT':'Thr',
        'AAC':'Asn', 'AAT':'Asn', 'AAA':'Lys', 'AAG':'Lys',
        'AGC':'Ser', 'AGT':'Ser', 'AGA':'Arg', 'AGG':'Arg',
        'CTA':'Leu', 'CTC':'Leu', 'CTG':'Leu', 'CTT':'Leu',
        'CCA':'Pro', 'CCC':'Pro', 'CCG':'Pro', 'CCT':'Pro',
        'CAC':'His', 'CAT':'His', 'CAA':'Gln', 'CAG':'Gln',
        'CGA':'Arg', 'CGC':'Arg', 'CGG':'Arg', 'CGT':'Arg',
        'GTA':'Val', 'GTC':'Val', 'GTG':'Val', 'GTT':'Val',
        'GCA':'Ala', 'GCC':'Ala', 'GCG':'Ala', 'GCT':'Ala',
        'GAC':'Asp', 'GAT':'Asp', 'GAA':'Glu', 'GAG':'Glu',
        'GGA':'Gly', 'GGC':'Gly', 'GGG':'Gly', 'GGT':'Gly',
        'TCA':'Ser', 'TCC':'Ser', 'TCG':'Ser', 'TCT':'Ser',
        'TTC':'Phe', 'TTT':'Phe', 'TTA':'Leu', 'TTG':'Leu',
        'TAC':'Tyr', 'TAT':'Tyr', 'TAA':'___', 'TAG':'___',
        'TGC':'Cys', 'TGT':'Cys', 'TGA':'___', 'TGG':'Trp',
    }
    
    return genetic_code[codon] ? genetic_code[codon] : "???"
}

function apply_changes(codon, changes) {
    let result = codon
    
    let meaning = {
        "CDf": ["C", "T"],
        "CDr": ["G", "A"],
        "ADf": ["A", "G"],
        "ADr": ["T", "C"]
    }

    
    for(let change of changes) {
        result = result.replaceAll(meaning[change][0], meaning[change][1].toLowerCase())
    }
    
    return result.toUpperCase()
}

function process_sequence(sequence, changes) {
    let codons = getCodons(sequence)
    let results = []
    
    // Generate all possible combinations of changes
    for(let codon of codons) results.push(apply_changes(codon, changes))
    
    return results
}

function getCodons(sequence) {
    if(typeof sequence == "object") return
    
    sequence = sequence.toUpperCase() // tsu.js -> String.clear()
    if(sequence.includes('>')) {
        sequence = sequence.split('\n')
        let title = sequence.shift()
        sequence = sequence.join('')
    }
    
    return sequence.match(/.{1,3}/g)
}

function getSeqTitle(sequence) {
    let length = getCodons(sequence).join('').length
    let res = `Unknown DNA sequence (${length}bp)`
    if(sequence.includes('>')) {
        res = sequence.split('\n').shift().replace('>', '')
    }

    return res
}

function seqAn() {
    let input = document.querySelector('#seqan-input').value
    if(!input.length) return

    let transitions = [
        document.getElementById("effect1").value,
        document.getElementById("effect2").value
    ]

    let res = process_sequence(input, transitions)
    document.querySelector('#seqan-res').innerHTML = compareSeqences(input, res)
    document.querySelector('#seqan-res').classList.remove('hidden')
    let newTitle = getSeqTitle(input) + " => " + JSON.stringify(transitions)
    document.querySelector('#seqan-res').setAttribute('data', `>${newTitle}\n${res.join('')}`)
    document.querySelector('#seqan-title').innerHTML = newTitle
}

function compareSeqences(input, output) {
    
    input = getCodons(input)
    let boxSize = 4
    
    let ruler = ""
    let inputseq = ""
    let inputaa = ""
    let transline = ""
    let outputseq = ""
    let outputaa = ""

    for(let ci=0; ci < input.length; ci++) {
        ruler += `${ci + 1}`.padEnd(boxSize, " ")
        let codonIn = `${input[ci]}`.padEnd(boxSize, " ")
        inputseq += codonIn
        let aaIn = translate_codon(input[ci]).padEnd(boxSize, " ")
        inputaa += aaIn
        
        if(input[ci] !== output[ci]) {
            let aaOut = translate_codon(output[ci]).padEnd(boxSize, " ")
            let bOpen = aaIn == aaOut ? `<span class="grey">` : ''
            let bClose = aaIn == aaOut ? `</span>` : ''
            
            let codonOut = `${output[ci]}`.padEnd(boxSize, " ")
            outputseq += bOpen + diffCodons(codonIn, codonOut) + bClose
            outputaa += bOpen + aaOut + bClose
            transline += bOpen + " ^ ".padEnd(boxSize, " ") + bClose
        } else {
            outputseq += "".padEnd(boxSize, " ")
            outputaa += "".padEnd(boxSize, " ")
            transline += "".padEnd(boxSize, " ")
        }
    }

    return `<div>${ruler}\n${inputseq}\n${inputaa}</div><div>${transline}\n${outputseq}\n${outputaa}</div>`
}

function diffCodons(cA, cB) {
    let string = ""
    for(let i=0; i < cA.length; i++) {
        if(cA[i] == cB[i]) string += cA[i]
        else string += `<u>${cB[i]}</u>`
    }
    return string
}

function clearSeqAn() {
    document.querySelector('#seqan-input').value = ""
    document.querySelector('#seqan-res').innerHTML = ""
    document.querySelector('#seqan-res').classList.add('hidden')
    document.querySelector('#seqan-title').innerHTML = ""
    document.querySelector('#seqan-res').setAttribute('data', '')
}

function switchSeqAn() {
    let value = document.querySelector('#seqan-res').getAttribute('data')
    if(!value || value.length == 0) return alert('No "result" sequence to use as new input.')
    clearSeqAn()
    document.querySelector('#seqan-input').value = value
}