document.addEventListener('DOMContentLoaded', () => {
    const tocList = document.getElementById('toc-list');
    const headings = document.querySelectorAll('.content h1, .content h2, .content h3');

    // Populate TOC dynamically
    headings.forEach((heading, index) => {
        const id = heading.id || `heading-${index}`;
        heading.id = id;

        const listItem = document.createElement('li');
        const link = document.createElement('a');
        link.href = `#${id}`;
        link.textContent = heading.textContent;
        link.setAttribute('data-level', heading.tagName.toLowerCase());

        listItem.appendChild(link);
        tocList.appendChild(listItem);
    });

    // Intersection Observer for active TOC links
    const observer = new IntersectionObserver(
        (entries) => {
            entries.forEach((entry) => {
                const tocLinks = tocList.querySelectorAll('a');
                tocLinks.forEach((link) => {
                    if (link.getAttribute('href').substring(1) === entry.target.id) {
                        link.classList.toggle('active', entry.isIntersecting);
                    }
                });
            });
        },
        { threshold: 0.5 }
    );

    headings.forEach((heading) => observer.observe(heading));

    // Smooth scrolling and offset adjustment for TOC links
    const tocLinks = tocList.querySelectorAll('a');
    tocLinks.forEach((link) => {
        link.addEventListener('click', (event) => {
            event.preventDefault(); // Prevent default anchor link behavior

            const targetId = link.getAttribute('href').substring(1);
            const targetElement = document.getElementById(targetId);

            if (targetElement) {
                const navbarHeight = document.querySelector('.navbar').offsetHeight || 70; // Height of the navbar
                const elementPosition = targetElement.getBoundingClientRect().top + window.scrollY;
                const offsetPosition = elementPosition - navbarHeight;

                window.scrollTo({
                    top: offsetPosition,
                    behavior: 'smooth', // Smooth scrolling
                });
            }
        });
    });
});

document.addEventListener("scroll", () => {
    const toc = document.querySelector(".toc");
    const scrollY = window.scrollY;
    const viewportHeight = window.innerHeight;
    const documentHeight = document.documentElement.scrollHeight;
  
    // Show TOC when past the first screen but not at the bottom
    if (scrollY > viewportHeight && scrollY + viewportHeight < documentHeight - 200) {
      toc.style.opacity = "1";
      toc.style.visibility = "visible";
    } else {
      toc.style.opacity = "0";
      toc.style.visibility = "hidden";
    }
  });
  
  